
#define STRICT
#include <windows.h>
#include <windowsx.h>
#include <math.h>
#include <stdio.h>


// 
char const szClassName[] = "Movement";
//   
char const szWindowTitle[] = "Korneev V.  Move, drawing in memory"
       "    Sea   and   Ship";


// 
BOOL RegisterApp(HINSTANCE hInst);
HWND CreateApp(HINSTANCE, int);
LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
void Move_OnCreate();
void Move_OnDestroy(HWND hwnd);
void Move_OnTimer(HWND hwnd);
void Move_OnKeydown(HWND hwnd);
void Move_OnPaint(HWND);
void Land(HDC hdc);
void See(HDC hdc, double p);
//    
struct Point
{
	double x;
	double y;
};
void Arrow(HDC hdc, Point *Pt, int N, double x1,double y1,
           double x2,double y2,double fi,double kx,double ky);
void TrcB(HDC hdc, double te, double dt);
void LineSun(HDC hdc, double x, double t);
// 
//#pragma argsused
int PASCAL WinMain(HINSTANCE hInst,      //() 
                   HINSTANCE hPrevInstance,
                   LPSTR     lpszCmdParam,
                   int       nCmdShow)
{
	MSG msg;                         //    

	if(!RegisterApp(hInst))          // 
		return FALSE;

	if(!CreateApp(hInst,nCmdShow))   // 
		return FALSE;

	while(GetMessage(&msg,NULL,0,0))  //   
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	return msg.wParam;
}

//  
BOOL RegisterApp(HINSTANCE hInst)
{
	WNDCLASS wc;

	wc.style         = CS_HREDRAW | CS_VREDRAW;     //  
	wc.lpfnWndProc   = WndProc;                     //  
	wc.cbClsExtra    = 0;
	wc.cbWndExtra    = 0;                       
	wc.hInstance     = hInst;                       // 
	wc.hIcon         = LoadIcon(NULL, IDI_APPLICATION); // 
	wc.hCursor	 = LoadCursor(NULL, IDC_ARROW);     // 
	wc.hbrBackground = GetStockBrush(GRAY_BRUSH);   //  
	wc.lpszMenuName  = "APP_MENU";                 //  
	wc.lpszClassName = szClassName;                 // 

	return RegisterClass(&wc);           //  
}


//   
HWND CreateApp(HINSTANCE hInstance, int nCmdShow)
{
	 HWND hwnd;   //  

	 hwnd = CreateWindow(szClassName,      //   
		 	 szWindowTitle,        //  
			 WS_OVERLAPPEDWINDOW,  //  
			 CW_USEDEFAULT,        //x -   
			 CW_USEDEFAULT,        //y -   
			 CW_USEDEFAULT,        //  
			 CW_USEDEFAULT,        //  
			 NULL,          
			 NULL,          
			 hInstance,     
			 NULL);         

	if(hwnd == NULL)
		return hwnd;

	ShowWindow(hwnd,nCmdShow);   // 
	UpdateWindow(hwnd);          //  

	return hwnd;
}

//   
LRESULT CALLBACK  WndProc(HWND hwnd, UINT msg,
			WPARAM wParam, LPARAM lParam)
{

	switch(msg)
	{
		case WM_CREATE:
			Move_OnCreate();
			break;
		case WM_PAINT:
			Move_OnPaint(hwnd);
			break;
		case WM_TIMER:
			Move_OnTimer(hwnd);
			break;
		case WM_KEYDOWN:
			Move_OnKeydown(hwnd);
			break;
		case WM_DESTROY:
			Move_OnDestroy(hwnd);
			break;
		default:
			return DefWindowProc(hwnd,msg,wParam,lParam);
	}

	return 0L;

}


//      
double xLeft, xRight, yBottom, yTop;
//        
int    nLeft, nRight, mBottom, mTop;


//  x   n
inline int xn(double x)
{
	return (int)((x - xLeft)/(xRight - xLeft)*(nRight - nLeft)) + nLeft;
}

//  y   m
inline int ym(double y)
{
	return (int)((y - yBottom)/(yTop - yBottom)*(mTop - mBottom)) + mBottom;
}


//      
//  
class Matrix2D
{

public:
	double x[3][3];

	Matrix2D();                               // 

	void rotate(double fi);                      // 
	void displa(double kx, double ky);           // 
	void transl(double tx, double ty );          // 

	// 
	friend Matrix2D operator * (const Matrix2D&, const Matrix2D&);

};

Matrix2D::Matrix2D()
{
	x[0][0] =  1; x[0][1] =  0; x[0][2] =  0;
	x[1][0] =  0; x[1][1] =  1; x[1][2] =  0;
	x[2][0] =  0; x[2][1] =  0; x[2][2] =  1;
}


void Matrix2D::rotate(double fi)
{
	double cosfi = cos(fi), sinfi = sin(fi);
	x[0][0] = cosfi; x[0][1] = -sinfi; x[0][2] = 0;
	x[1][0] = sinfi; x[1][1] =  cosfi; x[1][2] = 0;
	x[2][0] =     0; x[2][1] =      0; x[2][2] = 1;
}

void Matrix2D::displa(double kx, double ky)
{
	x[0][0] = kx; x[0][1] =  0; x[0][2] = 0;
	x[1][0] =  0; x[1][1] = ky; x[1][2] = 0;
	x[2][0] =  0; x[2][1] =  0; x[2][2] = 1;
}

void Matrix2D::transl(double tx, double ty )
{
	x[0][0] =  1; x[0][1] =  0; x[0][2] = tx;
	x[1][0] =  0; x[1][1] =  1; x[1][2] = ty;
	x[2][0] =  0; x[2][1] =  0; x[2][2] =  1;
}


Matrix2D operator * (const Matrix2D& a, const Matrix2D& b)
{
	Matrix2D c;
	double s;
	for(int i=0; i<3; i++)
		for(int j=0; j<3; j++)
		{
			s = 0;
			for(int n=0; n<3; n++)
				s += a.x[i][n]*b.x[n][j];
			c.x[i][j] = s;
		}
	return c;
}




//    P  
// A
Point affine( Matrix2D A,  Point P)
{
	Point Pp;
		Pp.x = A.x[0][0]*P.x + A.x[0][1]*P.y + A.x[0][2]*1;
		Pp.y = A.x[1][0]*P.x + A.x[1][1]*P.y + A.x[1][2]*1;
	return Pp;
}




//     
Point	PtS[8], PtS0[8];

//     
Point	PtB[11], PtB0[11];

//""    
	double tB , dtB0, dtB;

//""    
	double tS , dtS0, dtS;

//""    
	double tC , dtC0, dtC;

//""    
	double tM , dtM0, dtM;

//     
	double fi, kx, ky;

// 6 
	Matrix2D R, T, D, A, T1, T2;

//   
	double xB0, yB0;

//   
	double xS0, yS0;

//   
	double xC0, yC0;

//   
	double xM0, yM0;


//   
	double xB2, yB2, xB1, yB1;

//   
	double xS2, yS2, xS1, yS1;

//   
	double xC, yC;

//   
	double xM, yM;

//  
	double xStar[50], yStar[50];


//    
int MoveStop;


//  
UINT uElapse;


//    WM_CREATE
void Move_OnCreate()
{
//       
	xLeft = -10; xRight =  10; yBottom =  -6.0; yTop =  6.0;
	nLeft = 50; nRight  = 650; mTop  = 50; mBottom = 400;

//  
   PtS0[0].x =  0.0; PtS0[0].y =  6.0;
   PtS0[1].x =  4.5; PtS0[1].y =  4.5;
   PtS0[2].x =  6.0; PtS0[2].y =  0.0;
   PtS0[3].x =  4.5; PtS0[3].y = -4.5;
   PtS0[4].x =  0.0; PtS0[4].y = -6.0;
   PtS0[5].x = -4.5; PtS0[5].y = -4.5;
   PtS0[6].x = -6.0; PtS0[6].y =  0.0;
   PtS0[7].x = -4.5; PtS0[7].y =  4.5;

//  
   PtB0[0].x =  0.0;  PtB0[0].y = 14.0;
   PtB0[1].x =  6.0;  PtB0[1].y =  2.0;
   PtB0[2].x =  0.0;  PtB0[2].y =  5.0;
   PtB0[3].x =  0.0;  PtB0[3].y =  3.0;
   PtB0[4].x =  3.0;  PtB0[4].y = -0.0;
   PtB0[5].x = -4.0;  PtB0[5].y =  7.0;
   PtB0[6].x =  1.0;  PtB0[6].y = -1.0;
   PtB0[7].x =  5.0;  PtB0[7].y =  1.0;
   PtB0[8].x =  4.0;  PtB0[8].y = -2.5;
   PtB0[9].x =  2.5;  PtB0[9].y = -3.5;
  PtB0[10].x = -3.0; PtB0[10].y =  4.0;


//     
      for(int k=0; k<50; k++)
      {
         xStar[k] = xLeft+(xRight-xLeft)*(1.0*rand()/RAND_MAX);
         yStar[k] = yTop*((1.0*rand()/RAND_MAX));
      }


//      
	fi = 0; kx = 0.993; ky = 0.993;

//    
   dtB0 = 0.04;

//    
   dtS0 = 0.23;

//    
   dtC0 = 0.035;

//    
   dtM0 = 0.2;

//  
	uElapse = 70;

//   
	MoveStop = 1;
}



//    WM_DESTROY
void Move_OnDestroy(HWND hwnd)
{
	KillTimer(hwnd, 1);
	PostQuitMessage(0);   // 
}





// 
Point TrajB(double t)
{
	Point P;
   P.x = -(t - 4.5);
   P.y = (t - 4.5);
   return P;
}



//     Pt
void Boat(HDC hdc, Point *Pt, int N, double x1,double y1,
           double x2,double y2,double fi,double kx,double ky)
{

//   
		T1.transl(-x1,-y1);
		R.rotate(fi);
		D.displa(kx, ky);
		T.transl(x2-x1,y2-y1);
		T2.transl(x1,y1);

		A = T2*T*D*R*T1;

//    
		for(int n=0; n<N; n++)
			Pt[n] = affine(A,Pt[n]);

	POINT pt[7];
	pt[0].x = xn(Pt[5].x); pt[0].y = ym(Pt[5].y);
	pt[1].x = xn(Pt[6].x); pt[1].y = ym(Pt[6].y);
	pt[2].x = xn(Pt[7].x); pt[2].y = ym(Pt[7].y);

	HBRUSH hbrushOld;
	HPEN hpenOld;
	HBRUSH hbrush1, hbrush2, hbrush3, hbrush4;
	HPEN hpen1, hpen2, hpen3, hpen4, hpen5;

	hbrush1 = CreateSolidBrush(RGB(200,200,200));
	hbrushOld = (HBRUSH)SelectObject(hdc,hbrush1);     //  hbrush
	hpen1 = CreatePen(PS_SOLID,2,RGB(170,170,170));
	hpenOld = (HPEN)SelectObject(hdc,hpen1);         //   hpen

	Polygon(hdc,pt,3);


	pt[0].x = xn(Pt[5].x);  pt[0].y = ym(Pt[5].y);
	pt[1].x = xn(Pt[6].x);  pt[1].y = ym(Pt[6].y);
	pt[2].x = xn(Pt[9].x);  pt[2].y = ym(Pt[9].y);
	pt[3].x = xn(Pt[10].x); pt[3].y = ym(Pt[10].y);


	hbrush2 = CreateSolidBrush(RGB(100,80,80));
	SelectObject(hdc,hbrush2);     //  hbrush
	hpen2 = CreatePen(PS_SOLID,2,RGB(100,100,100));
	SelectObject(hdc,hpen2);         //   hpen

	Polygon(hdc,pt,4);


	pt[0].x = xn(Pt[6].x);  pt[0].y = ym(Pt[6].y);
	pt[1].x = xn(Pt[7].x);  pt[1].y = ym(Pt[7].y);
	pt[2].x = xn(Pt[8].x);  pt[2].y = ym(Pt[8].y);
	pt[3].x = xn(Pt[9].x);  pt[3].y = ym(Pt[9].y);


	hbrush3 = CreateSolidBrush(RGB(100,130,100));
	SelectObject(hdc,hbrush3);     //  hbrush
	hpen3 = CreatePen(PS_SOLID,2,RGB(120,120,120));
	SelectObject(hdc,hpen3);         //   hpen

	Polygon(hdc,pt,4);


	pt[0].x = xn(Pt[0].x);  pt[0].y = ym(Pt[0].y);
	pt[1].x = xn(Pt[1].x);  pt[1].y = ym(Pt[1].y);
	pt[2].x = xn(Pt[2].x);  pt[2].y = ym(Pt[2].y);


	hbrush4 = CreateSolidBrush(RGB(230,230,230));
	SelectObject(hdc,hbrush4);     //  hbrush
	hpen4 = CreatePen(PS_SOLID,2,RGB(200,200,0));
	SelectObject(hdc,hpen4);         //   hpen

	Polygon(hdc,pt,3);


	pt[0].x = xn(Pt[0].x);  pt[0].y = ym(Pt[0].y);
	pt[1].x = xn(Pt[3].x);  pt[1].y = ym(Pt[3].y);


	hpen5 = CreatePen(PS_SOLID,3,RGB(255,100,100));
	SelectObject(hdc,hpen5);         //   hpen

	MoveToEx(hdc, pt[0].x, pt[0].y, 0);
	LineTo(hdc,pt[1].x, pt[1].y);




   SelectObject(hdc,hpenOld);
	DeleteObject(hpen1);           //  hpen
	DeleteObject(hpen2);           //  hpen
	DeleteObject(hpen3);           //  hpen
	DeleteObject(hpen4);           //  hpen
	DeleteObject(hpen5);           //  hpen
   SelectObject(hdc,hbrushOld);
	DeleteObject(hbrush1);           //  hpen
	DeleteObject(hbrush2);           //  hpen
	DeleteObject(hbrush3);           //  hpen
	DeleteObject(hbrush4);           //  hpen

}


//  
void TracB(HDC hdc, double te, double dt)
{
	double x, y, t;

	t = 0;
	x = TrajB(t).x;
	y = TrajB(t).y;

	HPEN hpen, hpenOld;

	hpen = CreatePen(PS_SOLID,2,RGB(255,255,200));
	hpenOld = (HPEN)SelectObject(hdc,hpen);         //   hpen

	MoveToEx(hdc,xn(x),ym(y),0);
	do{
		t += dt;
		x = TrajB(t).x;
		y = TrajB(t).y;
		LineTo(hdc,xn(x),ym(y));
	}while(t < te);

   SelectObject(hdc,hpenOld);
	DeleteObject(hpen);           //  hpen

}




// 
Point TrajS(double t)
{
	Point P;
   P.x = -4.5*sin(0.1*t)+4.5;
   P.y = 4.5*cos(0.1*t);
   return P;
}



//     Pt
void Sun(HDC hdc, Point *Pt, int N, double x1,double y1,
           double x2,double y2,double fi,double kx,double ky)
{

//   
		T1.transl(-x1,-y1);
		R.rotate(fi);
		D.displa(kx, ky);
		T.transl(x2-x1,y2-y1);
		T2.transl(x1,y1);

		A = T2*T*D*R*T1;

//    
		for(int n=0; n<N; n++)
			Pt[n] = affine(A,Pt[n]);

	POINT pt[8];
	for(int i = 0; i < N; i++)
	{
      		pt[i].x = xn(Pt[i].x); pt[i].y = ym(Pt[i].y);
	}

	HBRUSH hbrush, hbrushOld;
	HPEN hpen, hpenOld;


	hbrush = CreateSolidBrush(RGB(255,255,0));
	hbrushOld = (HBRUSH)SelectObject(hdc,hbrush);     //  hbrush
	hpen = CreatePen(PS_SOLID,2,RGB(255,255,0));
	hpenOld = (HPEN)SelectObject(hdc,hpen);         //   hpen

	for(i = 0; i < 4; i++)
   {
   	MoveToEx(hdc,pt[i].x,pt[i].y,0);
   	LineTo(hdc,pt[i+4].x,pt[i+4].y);
   }

   Ellipse(hdc,pt[7].x,pt[7].y,pt[3].x,pt[3].y);




   SelectObject(hdc,hpenOld);
	DeleteObject(hpen);           //  hpen
   SelectObject(hdc,hbrushOld);
	DeleteObject(hbrush);           //  hpen

}



//  
void LineSun(HDC hdc, double x, double t)
{


	POINT pt[4];
	pt[0].x = xn(x+0.2*(1+t)); pt[0].y = ym(-0.1);
	pt[1].x = xn(x-0.2*(1+t)); pt[1].y = ym(-0.1);
	pt[2].x = xn(-0.7*(1+t));  pt[2].y = ym(yBottom+0.1);
	pt[3].x = xn(0.7*(1+t));   pt[3].y = ym(yBottom+0.1);

	HBRUSH hbrush, hbrushOld;
	HPEN hpen, hpenOld;

   int colR, colG, colB;
   colR = (int)(255*exp(-0.1*t));
   colG = (int)(155*exp(-0.15*t)+ 100/(1+0.1*t));
   colB = (int)((1-exp(-0.2*t))*230/(1+0.1*t));
	hbrush = CreateSolidBrush(RGB(colR, colG, colB));
	hbrushOld = (HBRUSH)SelectObject(hdc,hbrush);     //  hbrush
	hpen = CreatePen(PS_SOLID,2,RGB(colR, colG, colB));
	hpenOld = (HPEN)SelectObject(hdc,hpen);         //   hpen

   Polygon(hdc,pt,4);

   SelectObject(hdc,hpenOld);
	DeleteObject(hpen);           //  hpen
   SelectObject(hdc,hbrushOld);
	DeleteObject(hbrush);           //  hpen
}




// 
Point TrajC(double t)
{
	Point P;
   P.x = t-7.5;
   P.y = 4.0;
   return P;
}



// 
void Cloud(HDC hdc, double x0,double y0,double t)
{


	HBRUSH hbrush, hbrushOld;
	HPEN hpen, hpenOld;

   int col1 = (int)(255*exp(-0.1*t));

	hbrush = CreateSolidBrush(RGB(col1,col1,col1));
	hbrushOld = (HBRUSH)SelectObject(hdc,hbrush);     //  hbrush
	hpen = CreatePen(PS_SOLID,2,RGB(col1,col1,col1));
	hpenOld = (HPEN)SelectObject(hdc,hpen);         //   hpen

   int x1,y1,x2,y2;
   x1 = xn(x0-2); y1 = ym(y0+1);
   x2 = xn(x0+2); y2 = ym(y0-1);

   Ellipse(hdc,x1,y1,x2,y2);


   x1 = xn(x0+1-1); y1 = ym(y0+0.5+1);
   x2 = xn(x0+1+1); y2 = ym(y0+0.5-1);

   Ellipse(hdc,x1,y1,x2,y2);


   SelectObject(hdc,hpenOld);
	DeleteObject(hpen);           //  hpen
   SelectObject(hdc,hbrushOld);
	DeleteObject(hbrush);           //  hpen

}



// 
Point TrajM(double t)
{
	Point P;
   P.x = -6.5;
   P.y = 0.1*t-3;
   return P;
}



// 
void Moon(HDC hdc, double x0,double y0,double t)
{


	HBRUSH hbrush, hbrushOld;
	HPEN hpen, hpenOld;

   int col = (int)(255*(1-exp(-0.2*t)));

	hbrush = CreateSolidBrush(RGB(col,col,col));
	hbrushOld = (HBRUSH)SelectObject(hdc,hbrush);     //  hbrush
	hpen = CreatePen(PS_SOLID,2,RGB(col,col,col));
	hpenOld = (HPEN)SelectObject(hdc,hpen);         //   hpen

   int x1,y1,x2,y2;
   x1 = xn(x0-1); y1 = ym(y0+0.8);
   x2 = xn(x0+1); y2 = ym(y0-0.8);

   Ellipse(hdc,x1,y1,x2,y2);




   SelectObject(hdc,hpenOld);
	DeleteObject(hpen);           //  hpen
   SelectObject(hdc,hbrushOld);
	DeleteObject(hbrush);           //  hpen

}



//  
void LineMoon(HDC hdc, double x, double t)
{


	double d1 = 1*(1-exp(-0.1*(t-20))), d2 = 4*(1-exp(-0.1*(t-20)));

	POINT pt[4];
	pt[0].x = xn(x+d1); pt[0].y = ym(-0.1);
	pt[1].x = xn(x-d1); pt[1].y = ym(-0.1);
	pt[2].x = xn(-d2);  pt[2].y = ym(yBottom+0.1);
	pt[3].x = xn(d2);   pt[3].y = ym(yBottom+0.1);

	HBRUSH hbrush, hbrushOld;
	HPEN hpen, hpenOld;

   int col = 255;
	hbrush = CreateSolidBrush(RGB(col, col, col));
	hbrushOld = (HBRUSH)SelectObject(hdc,hbrush);     //  hbrush
	hpen = CreatePen(PS_SOLID,2,RGB(col, col, col));
	hpenOld = (HPEN)SelectObject(hdc,hpen);         //   hpen

   Polygon(hdc,pt,4);

   SelectObject(hdc,hpenOld);
	DeleteObject(hpen);           //  hpen
   SelectObject(hdc,hbrushOld);
	DeleteObject(hbrush);           //  hpen
}



// 
void Star(HDC hdc, double t)
{


   int colR = (int)(255*(1-exp(-0.001*t*t)));
   int colG = (int)(255*(1-exp(-0.001*t*t)));
   int colB = (int)(255*(1-exp(-0.001*t*t)));

	double x0, y0;
	int x1,y1;

	for(int k=0; k<50; k++)
	{
		x0 = xStar[k];  y0 = yStar[k];

		x1 = xn(x0); y1 = ym(y0);

		SetPixel(hdc,x1,y1,RGB(colR,colG,colB));
		SetPixel(hdc,x1-1,y1,RGB(colR,colG,colB));
		SetPixel(hdc,x1-2,y1,RGB(colR,colG,colB));
		SetPixel(hdc,x1+1,y1,RGB(colR,colG,colB));
		SetPixel(hdc,x1+2,y1,RGB(colR,colG,colB));
		SetPixel(hdc,x1,y1-1,RGB(colR,colG,colB));
		SetPixel(hdc,x1,y1-2,RGB(colR,colG,colB));
		SetPixel(hdc,x1,y1+1,RGB(colR,colG,colB));
		SetPixel(hdc,x1,y1+2,RGB(colR,colG,colB));


	}

}




// 
void Sky(HDC hdc, double t)
{

	HBRUSH hbrush, hbrushOld;
	HPEN hpen, hpenOld;

   int colR = (int)(200*exp(-0.004*t*t));
   int colG = (int)(255*exp(-0.01*t*t));
   int colB = (int)(255*exp(-0.01*t*t));

	hbrush = CreateSolidBrush(RGB(colR,colG,colB));
	hbrushOld = (HBRUSH)SelectObject(hdc,hbrush);     //  hbrush
	hpen = CreatePen(PS_SOLID,1,RGB(colR,colG,colB));
	hpenOld = (HPEN)SelectObject(hdc,hpen);         //   hpen

 	Rectangle(hdc,nLeft,mBottom,nRight,mTop);


   SelectObject(hdc,hpenOld);
	DeleteObject(hpen);           //  hpen
   SelectObject(hdc,hbrushOld);
	DeleteObject(hbrush);           //  hpen
}



// 
void See(HDC hdc, double t)
{

	HBRUSH hbrush, hbrushOld;
	HPEN hpen1, hpen2, hpenOld;

   int colG = (int)(100/(1+0.1*t));
   int colB = (int)(230/(1+0.1*t));

	hbrush = CreateSolidBrush(RGB(0,colG,colB));
	hbrushOld = (HBRUSH)SelectObject(hdc,hbrush);     //  hbrush
	hpen1 = CreatePen(PS_SOLID,1,RGB(0,colG,colB));
	hpenOld = (HPEN)SelectObject(hdc,hpen1);         //   hpen

 	Rectangle(hdc,nLeft,mBottom,nRight,(mTop+mBottom)/2);

	hpen2 = CreatePen(PS_SOLID,1,RGB(0,255,255));
   SelectObject(hdc,hpen2);

	MoveToEx(hdc, nLeft,(mTop+mBottom)/2, 0);
	LineTo(hdc,nRight,(mTop+mBottom)/2);

   SelectObject(hdc,hpenOld);
	DeleteObject(hpen1);           //  hpen
	DeleteObject(hpen2);           //  hpen
   SelectObject(hdc,hbrushOld);
	DeleteObject(hbrush);           //  hpen
}





//   WM_KEYDOWN
void Move_OnKeydown(HWND hwnd)
{
	if(MoveStop == 0)
	{
		KillTimer(hwnd, 1);
		MoveStop = 1;
	}
	else
	{
		SetTimer(hwnd, 1, uElapse, NULL);
		MoveStop = 0;
	}
}













//    WM_PAINT
void Move_OnPaint(HWND hwnd)
{

	HDC hdc;
	PAINTSTRUCT ps;

	hdc = BeginPaint(hwnd, &ps);  //  


	HBRUSH hbrush, hbrushOld;
	HPEN hpen, hpenOld;

	hbrush = CreateSolidBrush(RGB(0,255,255));
	hbrushOld = (HBRUSH)SelectObject(hdc,hbrush);     //  hbrush
	hpen = CreatePen(PS_SOLID,3,RGB(255,255,255));
	hpenOld = (HPEN)SelectObject(hdc,hpen);         //   hpen

 	Rectangle(hdc,nLeft,mBottom,nRight,mTop);

   SelectObject(hdc,hpenOld);
	DeleteObject(hpen);           //  hpen
   SelectObject(hdc,hbrushOld);
	DeleteObject(hbrush);           //  hpen

   Sky(hdc,0);

	SetTextColor(hdc,RGB(230,230,220));
	SetBkColor(hdc,RGB(0,0,0));
   TextOut(hdc, 225,100," P r e s s   a n y   k e y ",27);

//    
   tB = 0; dtB = dtB0;

//    
   tS = 0; dtS = dtS0;

//    
   tC = 0; dtC = dtC0;

//    
   tM = 0; dtM = dtM0;

//   
	xB0 = TrajB(tB).x;
	yB0 = TrajB(tB).y;

//   
	xS0 = TrajS(tS).x;
	yS0 = TrajS(tS).y;

//   
	xC0 = TrajC(tC).x;
	yC0 = TrajC(tC).y;

//   
	xM0 = TrajM(tM).x;
	yM0 = TrajM(tM).y;

	for(int n=0; n<8; n++)
   	PtS[n] = PtS0[n];

	for(n=0; n<11; n++)
   	PtB[n] = PtB0[n];

	Moon(hdc,xM0,yM0,0);
   See(hdc,0);
	Sun(hdc,PtS,8,0,0,xS0,yS0,0,0.2,0.2);
	LineSun(hdc,xS0,0);
	Cloud(hdc,xC0,yC0,0);
	Boat(hdc,PtB,11,0,0,xB0,yB0,0,0.3,0.3);

  	xB1 = xB0; yB1 = yB0;
  	xS1 = xS0; yS1 = yS0;

	EndPaint(hwnd, &ps);       //  

}



//    WM_TIMER
void Move_OnTimer(HWND hwnd)
{
	HDC hdcMem, hdcWin;
	HBITMAP hBitmap, hBitmapOld;
	hdcWin = GetDC(hwnd);  //  
	hdcMem = CreateCompatibleDC(hdcWin); // 
                     //    

   //     -     
   //   .       
	hBitmap = CreateCompatibleBitmap(hdcWin, nRight, mBottom); //
               //     
	hBitmapOld = (HBITMAP)SelectObject(hdcMem, hBitmap); //
                //     

		tS += dtS;
      dtS *= 0.9905;
		xS2 = TrajS(tS).x;
		yS2 = TrajS(tS).y;


   Sky(hdcMem,tS);
//    (x2, y2)
	Sun(hdcMem,PtS,8,xS1,yS1,xS2,yS2,fi,1.01,1.001);
	xS1 = xS2; yS1 = yS2;

		tM += dtM;
      dtM *= 0.997;
		xM = TrajM(tM).x;
		yM = TrajM(tM).y;

//    (x2, y2)

	if(yM>-1.0)
      	Star(hdcMem,tM);

   Moon(hdcMem, xM, yM, tM);

   See(hdcMem,tS);

	double xx;
   xx = xS2;
      if(yS2 < -0.0) xx = 0;

   if(yS2 > -1.0)
	  LineSun(hdcMem,xx,tS);
	if(yM > -1.0)
   	LineMoon(hdcMem,xM,tM);



		tC += dtC;
		xC = TrajC(tC).x;
		yC = TrajC(tC).y;

//    (x2, y2)
   Cloud(hdcMem, xC, yC, tC);




		tB += dtB;
      dtB *= 0.9913;
		xB2 = TrajB(tB).x;
		yB2 = TrajB(tB).y;

//    (x2, y2)
   TracB(hdcMem, tB, dtB);
	Boat(hdcMem,PtB,11,xB1,yB1,xB2,yB2,fi,kx,ky);
	xB1 = xB2; yB1 = yB2;

	BitBlt(hdcWin,nLeft,mTop,nRight,mBottom,hdcMem,nLeft,mTop,SRCCOPY); //-
   //     



	SelectObject(hdcMem, hBitmapOld); //  
	DeleteObject(hBitmap); //  
	DeleteDC(hdcMem);  //    
	ReleaseDC(hwnd,hdcWin);  //  
}


