

#define STRICT
#include <windows.h>
#include <windowsx.h>
#include <math.h>
#include <stdio.h>

// 
HINSTANCE hInstApp;
//    
HBITMAP TheBitmap;



// 
char const szClassName[] = "Movement";
//   
char const szWindowTitle[] = 
	"Korneev V.   Picture Field from bmp-file.  Move plain";


// 
BOOL RegisterApp(HINSTANCE hInst);
HWND CreateApp(HINSTANCE, int);
LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
void Move_OnCreate();
void Move_OnDestroy(HWND hwnd);
void Move_OnTimer(HWND hwnd);
void Move_OnKeydown(HWND hwnd);
void Move_OnPaint(HWND);


// 
//#pragma argsused
int PASCAL WinMain(HINSTANCE hInst,      //() 
                   HINSTANCE hPrevInstance,
                   LPSTR     lpszCmdParam,
                   int       nCmdShow)
{
	MSG msg;                         //    

	if(!RegisterApp(hInst))          // 
		return FALSE;

	if(!CreateApp(hInst,nCmdShow))   // 
		return FALSE;

	while(GetMessage(&msg,NULL,0,0))  //   
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	return msg.wParam;
}

//  
BOOL RegisterApp(HINSTANCE hInst)
{
	WNDCLASS wc;

	wc.style         = CS_HREDRAW | CS_VREDRAW;     //  
	wc.lpfnWndProc   = WndProc;                     //  
	wc.cbClsExtra    = 0;
	wc.cbWndExtra    = 0;                       
	wc.hInstance     = hInst;                       // 
	wc.hIcon         = LoadIcon(NULL, IDI_APPLICATION); // 
	wc.hCursor	 = LoadCursor(NULL, IDC_ARROW);     // 
	wc.hbrBackground = GetStockBrush(GRAY_BRUSH);   //  
	wc.lpszMenuName  = "APP_MENU";                  //  
	wc.lpszClassName = szClassName;                 // 

	return RegisterClass(&wc);           //  
}


//   
HWND CreateApp(HINSTANCE hInstance, int nCmdShow)
{

	hInstApp = hInstance;

	 HWND hwnd;   //  

	 hwnd = CreateWindow(szClassName,      //   
		 	 szWindowTitle,        //  
			 WS_OVERLAPPEDWINDOW,  //  
			 CW_USEDEFAULT,        //x -   
			 CW_USEDEFAULT,        //y -   
			 CW_USEDEFAULT,        //  
			 CW_USEDEFAULT,        //  
			 NULL,          
			 NULL,          
			 hInstance,     
			 NULL);         

	if(hwnd == NULL)
		return hwnd;

	ShowWindow(hwnd,nCmdShow);   // 
	UpdateWindow(hwnd);          //  

	return hwnd;
}

//   
LRESULT CALLBACK  WndProc(HWND hwnd, UINT msg,
			WPARAM wParam, LPARAM lParam)
{

	switch(msg)
	{
		case WM_CREATE:
			Move_OnCreate();
			break;
		case WM_PAINT:
			Move_OnPaint(hwnd);
			break;
		case WM_TIMER:
			Move_OnTimer(hwnd);
			break;
		case WM_KEYDOWN:
			Move_OnKeydown(hwnd);
			break;
		case WM_DESTROY:
			Move_OnDestroy(hwnd);
			break;
		default:
			return DefWindowProc(hwnd,msg,wParam,lParam);
	}

	return 0L;

}





//      
double xLeft, xRight, yBottom, yTop;
//        
int    nLeft, nRight, mBottom, mTop;


//  x   n
inline int xn(double x)
{
	return (int)((x - xLeft)/(xRight - xLeft)*(nRight - nLeft)) + nLeft;
}

//  y   m
inline int ym(double y)
{
	return (int)((y - yBottom)/(yTop - yBottom)*(mTop - mBottom)) + mBottom;
}


//      
//  
class Matrix2D
{

public:
	double x[3][3];

	Matrix2D();                               // 

	void rotate(double fi);                      // 
	void displa(double kx, double ky);           // 
	void transl(double tx, double ty );          // 

	// 
	friend Matrix2D operator * (const Matrix2D&, const Matrix2D&);

};

Matrix2D::Matrix2D()
{
	x[0][0] =  1; x[0][1] =  0; x[0][2] =  0;
	x[1][0] =  0; x[1][1] =  1; x[1][2] =  0;
	x[2][0] =  0; x[2][1] =  0; x[2][2] =  1;
}


void Matrix2D::rotate(double fi)
{
	double cosfi = cos(fi), sinfi = sin(fi);
	x[0][0] = cosfi; x[0][1] = -sinfi; x[0][2] = 0;
	x[1][0] = sinfi; x[1][1] =  cosfi; x[1][2] = 0;
	x[2][0] =     0; x[2][1] =      0; x[2][2] = 1;
}

void Matrix2D::displa(double kx, double ky)
{
	x[0][0] = kx; x[0][1] =  0; x[0][2] = 0;
	x[1][0] =  0; x[1][1] = ky; x[1][2] = 0;
	x[2][0] =  0; x[2][1] =  0; x[2][2] = 1;
}

void Matrix2D::transl(double tx, double ty )
{
	x[0][0] =  1; x[0][1] =  0; x[0][2] = tx;
	x[1][0] =  0; x[1][1] =  1; x[1][2] = ty;
	x[2][0] =  0; x[2][1] =  0; x[2][2] =  1;
}


Matrix2D operator * (const Matrix2D& a, const Matrix2D& b)
{
	Matrix2D c;
	double s;
	for(int i=0; i<3; i++)
		for(int j=0; j<3; j++)
		{
			s = 0;
			for(int n=0; n<3; n++)
				s += a.x[i][n]*b.x[n][j];
			c.x[i][j] = s;
		}
	return c;
}


//    
struct Point
{
	double x;
	double y;
};


//    P  
// A
Point affine( Matrix2D A,  Point P)
{
	Point Pp;
		Pp.x = A.x[0][0]*P.x + A.x[0][1]*P.y + A.x[0][2]*1;
		Pp.y = A.x[1][0]*P.x + A.x[1][1]*P.y + A.x[1][2]*1;
	return Pp;
}



//     
Point	PtA[7], PtA0[7];

//     
Point	PtP[7], PtP0[7];


//""     
	double tA , dtA, dtA0;

//""     
	double tP , dtP, dtP0;


//      
	double fiA, kxA, kyA;

//      
	double fiP1, fiP2, kxP, kyP;

// 6   
	Matrix2D R, R1, T, D, A, T1, T2;

//     
	double xA0, yA0, xA2, yA2, xA1, yA1;

//     
	double xP0, yP0, xP2, yP2, xP1, yP1;

//    
	int MoveStop;

//  
UINT uElapse;

//    WM_CREATE
void Move_OnCreate()
{
//       
	xLeft = -10; xRight =  10; yBottom =  -7.5; yTop =  7.5;
	nLeft = 60; nRight  = 700; mTop  = 15; mBottom = 495;

//   
   PtA0[0].x =  1.0; PtA0[0].y =  0.0;
   PtA0[1].x =  2.0; PtA0[1].y =  0.0;
   PtA0[2].x =  0.0; PtA0[2].y =  2.0;
   PtA0[3].x = -2.0; PtA0[3].y =  0.0;
   PtA0[4].x = -1.0; PtA0[4].y =  0.0;
   PtA0[5].x = -1.0; PtA0[5].y = -2.0;
   PtA0[6].x =  1.0; PtA0[6].y = -2.0;

//   
   PtP0[0].x =  6.0; PtP0[0].y =  0.0;
   PtP0[1].x =  4.0; PtP0[1].y =  2.0;
   PtP0[2].x = -3.0; PtP0[2].y =  0.0;
   PtP0[3].x = -4.0; PtP0[3].y =  2.0;
   PtP0[4].x = -6.0; PtP0[4].y =  2.0;
   PtP0[5].x = -6.0; PtP0[5].y = -1.0;
   PtP0[6].x =  4.0; PtP0[6].y = -2.0;


//      
	fiA = 3.14/70; kxA = 0.997; kyA = 0.997;

//  
	kxP = 0.994; kyP = 0.994;

//   
   dtA0 = 0.04;

//   
   dtP0 = 0.11;

//  
	uElapse = 50;

//  
	MoveStop = 1;

//    
//hInstApp -     
// 
//"Bitmap" -  
	TheBitmap=LoadBitmap(hInstApp, "Bitmap");

}



//   WM_KEYDOWN
void Move_OnKeydown(HWND hwnd)
{
	if(MoveStop == 0)
	{
		KillTimer(hwnd, 1);
		MoveStop = 1;
	}
	else
	{
		SetTimer(hwnd, 1, uElapse, NULL);
		MoveStop = 0;
	}
}


//    WM_DESTROY
void Move_OnDestroy(HWND hwnd)
{
	KillTimer(hwnd, 1);
	DeleteObject(TheBitmap);   //    
	PostQuitMessage(0);   //  
}

// 
Point TrajA(double t)
{
	Point P;
   P.x = t - 8;
   P.y = 4 - (t - 8)*(t - 8)/10;
   return P;
}

// 
Point TrajP(double t)
{
	Point P;
   P.x = -7.5 + 1.7*(t-1.5) - 4/(1+0.075*t)*sin(1.5*(t-1.5));
   P.y = 1 +0.25*(t-1.5)- 4/(1+0.075*t)*cos(1.5*(t-1.5));
   return P;
}



void Land(HDC hdc)
{

	HBRUSH hbrush, hbrushOld;
	HPEN hpen1, hpen2, hpenOld;

	hbrush = CreateSolidBrush(RGB(0,200,0));
	hbrushOld = (HBRUSH)SelectObject(hdc,hbrush);     //  hbrush
	hpen1 = CreatePen(PS_SOLID,3,RGB(255,255,255));
	hpenOld = (HPEN)SelectObject(hdc,hpen1);         //   hpen



   Ellipse(hdc,nLeft+160,mTop+20,nRight-160,mBottom-20);

 	//  
	hpen2 = CreatePen(PS_SOLID,1,RGB(0,255,255));
	SelectObject(hdc,hpen2);    //   hpen

	int nb, ne, mb, me;

	nb = xn(xLeft); mb = ym(0);
	MoveToEx(hdc, nb, mb, 0);
	ne = xn(xRight); me = ym(0);
	LineTo(hdc,ne,me);

	nb = xn(0); mb = ym(yBottom);
	MoveToEx(hdc, nb, mb, 0);
	ne = xn(0); me = ym(yTop);
	LineTo(hdc,ne,me);

   SelectObject(hdc,hpenOld);
	DeleteObject(hpen1);           //  hpen
	DeleteObject(hpen2);           //  hpen
   SelectObject(hdc,hbrushOld);
	DeleteObject(hbrush);           //  hpen
}


//     Pt   x2, y2
void Arrow(HDC hdc, Point *Pt, int N, double x1,double y1,
           double x2,double y2,double fi,double kx,double ky)
{

//   
		T1.transl(-x1,-y1);
		R.rotate(fi);
		D.displa(kx, ky);
		T.transl(x2-x1,y2-y1);
		T2.transl(x1,y1);

		A = T2*T*D*R*T1;

//    
		for(int n=0; n<7; n++)
			Pt[n] = affine(A,Pt[n]);


	POINT pt[7];
	for(int i = 0; i < N; i++)
	{
      		pt[i].x = xn(Pt[i].x); pt[i].y = ym(Pt[i].y);
	}

	HBRUSH hbrush, hbrushOld;
	HPEN hpen, hpenOld;

	hbrush = CreateSolidBrush(RGB(255,0,0));
	hbrushOld = (HBRUSH)SelectObject(hdc,hbrush);     //  hbrush
	hpen = CreatePen(PS_SOLID,2,RGB(0,0,255));
	hpenOld = (HPEN)SelectObject(hdc,hpen);         //   hpen

	Polygon(hdc,pt,N);

   SelectObject(hdc,hpenOld);
	DeleteObject(hpen);           //  hpen
   SelectObject(hdc,hbrushOld);
	DeleteObject(hbrush);           //  hpen

}


//  
void TracA(HDC hdc, double te, double dt)
{
	double x, y, t;

	t = 0;
	x = TrajA(t).x;
	y = TrajA(t).y;

	HPEN hpen, hpenOld;

	hpen = CreatePen(PS_SOLID,2,RGB(255,255,0));
	hpenOld = (HPEN)SelectObject(hdc,hpen);         //   hpen

	MoveToEx(hdc,xn(x),ym(y),0);
	do{
		t += dt;
		x = TrajA(t).x;
		y = TrajA(t).y;
		LineTo(hdc,xn(x),ym(y));
	}while(t < te);

   SelectObject(hdc,hpenOld);
	DeleteObject(hpen);           //  hpen

}


//     Pt   x2, y2
void Plain(HDC hdc, Point *Pt, int N, double x1,double y1,
           double x2,double y2,double fi1,double fi2,double kx,double ky)
{

//   
		T1.transl(-x1,-y1);
		R1.rotate(-fi1);
		R.rotate(fi2);
		D.displa(kx, ky);
		T.transl(x2-x1,y2-y1);
		T2.transl(x1,y1);

		A = T2*T*D*R*R1*T1;

//    
		for(int n=0; n<7; n++)
			Pt[n] = affine(A,Pt[n]);


	POINT pt[7];
	for(int i = 0; i < N; i++)
	{
      		pt[i].x = xn(Pt[i].x); pt[i].y = ym(Pt[i].y);
	}

	HBRUSH hbrush, hbrushOld;
	HPEN hpen, hpenOld;

	hbrush = CreateSolidBrush(RGB(0xD0,0xD0,0xC0));
	hbrushOld = (HBRUSH)SelectObject(hdc,hbrush);     //  hbrush
	hpen = CreatePen(PS_SOLID,1,RGB(0x70,0x70,0x70));
	hpenOld = (HPEN)SelectObject(hdc,hpen);         //   hpen

	Polygon(hdc,pt,N);

   SelectObject(hdc,hpenOld);
	DeleteObject(hpen);           //  hpen
   SelectObject(hdc,hbrushOld);
	DeleteObject(hbrush);           //  hpen

}

//  
void TracP(HDC hdc, double te, double dt)
{
	double x, y, t;

	t = 0;
	x = TrajP(t).x;
	y = TrajP(t).y;

	HPEN hpen, hpenOld;

	hpen = CreatePen(PS_SOLID,1,RGB(255,255,255));
	hpenOld = (HPEN)SelectObject(hdc,hpen);         //   hpen

	MoveToEx(hdc,xn(x),ym(y),0);
	do{
		t += dt;
		x = TrajP(t).x;
		y = TrajP(t).y;
		LineTo(hdc,xn(x),ym(y));
	}while(t < te);

   SelectObject(hdc,hpenOld);
	DeleteObject(hpen);           //  hpen

}



//    WM_PAINT
void Move_OnPaint(HWND hwnd)
{

	HDC hdc;
	PAINTSTRUCT ps;

	hdc = BeginPaint(hwnd, &ps);  //  


	HBRUSH hbrush, hbrushOld;
	HPEN hpen, hpenOld;

	hbrush = CreateSolidBrush(RGB(0,0,0));
	hbrushOld = (HBRUSH)SelectObject(hdc,hbrush);     //  hbrush
	hpen = CreatePen(PS_SOLID,3,RGB(255,255,255));
	hpenOld = (HPEN)SelectObject(hdc,hpen);         //   hpen

 	Rectangle(hdc,nLeft,mBottom,nRight,mTop);

   SelectObject(hdc,hpenOld);
	DeleteObject(hpen);           //  hpen
   SelectObject(hdc,hbrushOld);
	DeleteObject(hbrush);           //  hpen


	//   hdcMem     hdc
	HDC hdcMem = CreateCompatibleDC(hdc);
	//   TheBitmap    hdcMem
	HBITMAP hBitmapOld = (HBITMAP)SelectObject(hdcMem, TheBitmap); 
	
	//     hdcMem    hdc
	BitBlt(hdc, nLeft, mTop, nRight-nLeft, mBottom-mTop, hdcMem, 0, 0, SRCCOPY);

	
	SelectObject(hdcMem, hBitmapOld); //  
	DeleteDC(hdcMem);  //    




//   Land(hdc);

	SetTextColor(hdc,RGB(230,230,220));
	SetBkColor(hdc,RGB(0,0,0));
   TextOut(hdc, 225,100," P r e s s   a n y   k e y ",27);

//    
//   tA = 0; dtA = dtA0;

//   
//	xA0 = TrajA(tA).x;
//	yA0 = TrajA(tA).y;

//	for(int n=0; n<7; n++)
//   	PtA[n] = PtA0[n];

//    
//	Arrow(hdc,PtA,7,0,0,xA0,yA0,0,0.75,0.75);

//  	xA1 = xA0; yA1 = yA0;

//    
   tP = 0; dtP = dtP0;

//   
	xP0 = TrajP(tA).x;
	yP0 = TrajP(tA).y;

	for(int n=0; n<7; n++)
   	PtP[n] = PtP0[n];

//    
	Plain(hdc,PtP,7,0,0,xP0,yP0,0,0,0.3,0.3);

   fiP1 = 0;
  	xP1 = xP0; yP1 = yP0;



	EndPaint(hwnd, &ps);       //  

}



//    WM_TIMER
void Move_OnTimer(HWND hwnd)
{
	HDC hdcMem, hdcWin;
	HBITMAP hBitmap, hBitmapOld;
	//  
	hdcWin = GetDC(hwnd);  
	//      
	hdcMem = CreateCompatibleDC(hdcWin); 
                     
    //      -     
    //   .       
	
	// ()      
	hBitmap = CreateCompatibleBitmap(hdcWin, nRight, mBottom); 
    //   hBitmap    hdcMem          
	hBitmapOld = (HBITMAP)SelectObject(hdcMem, hBitmap); 
                
	//  TheBitmap      hdcMem1
	//       hdcMem.    
	//      
	//  
	
	//    hdcMem1
	//      hdcMem
	HDC hdcMem1 = CreateCompatibleDC(hdcMem); 
    //   TheBitmap    hdcMem1
	SelectObject(hdcMem1, TheBitmap); 
	//  TheBitmap   
	//  hdcMem1      hdcMem 
	BitBlt(hdcMem, nLeft,mTop, nRight-nLeft, mBottom-mTop, hdcMem1, 0,0, SRCCOPY);

//		tA += dtA;
//		xA2 = TrajA(tA).x;
//		if(xA2 >= xRight-1)
//		{
//			xA2 = xRight-1;
//			tA -= dtA;
//		}
//		yA2 = TrajA(tA).y;

//    (x2, y2)
//	Arrow(hdcMem,PtA,7,xA1,yA1,xA2,yA2,fiA,kxA,kyA);
//	TracA(hdcMem, tA, dtA);

//	xA1 = xA2; yA1 = yA2;

//   Land(hdcMem);

		tP += dtP;
		dtP *= 0.99115;
		xP2 = TrajP(tP).x;
		yP2 = TrajP(tP).y;

      fiP2 =atan2l(yP2-yP1,xP2-xP1);

//    (x2, y2)
	Plain(hdcMem,PtP,7,xP1,yP1,xP2,yP2,fiP1,fiP2,kxP,kyP);
	TracP(hdcMem, tP, dtP);

	xP1 = xP2; yP1 = yP2; fiP1 = fiP2;

	//     
	BitBlt(hdcWin, nLeft,mTop, nRight-nLeft, mBottom-mTop, hdcMem, nLeft,mTop, SRCCOPY);

	SelectObject(hdcMem, hBitmapOld); //  
	DeleteObject(hBitmap);           //  
	DeleteDC(hdcMem);                //    
	DeleteDC(hdcMem1);               //    
	ReleaseDC(hwnd,hdcWin);          //  
}






