

#define STRICT
#include <windows.h>
#include <windowsx.h>
#include <gl\gl.h>
#include <gl\glu.h>
#include <stdio.h>

//    
char const szClassName[] = "OpenGL";
// 
char const szWindowTitle[] = "Korneev V.  Graphics by OpenGL";


// 
BOOL RegisterApp(HINSTANCE);
HWND CreateApp(HINSTANCE, int);
LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
void DestroyOpenGL();
void DrawOpenGL(HWND);


// 
int PASCAL WinMain(HINSTANCE hInstance,
				HINSTANCE hPrevInstance,
				LPSTR     lpszCmdParam,
				int       nCmdShow)
{
	MSG msg;     //    

//    
	if(!RegisterApp(hInstance))
		return FALSE;

//   
	if(!CreateApp(hInstance,nCmdShow))
		return FALSE;

//  
	while(GetMessage(&msg,NULL,0,0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	return msg.wParam;
}

//    
BOOL RegisterApp(HINSTANCE hInstance)
{
	WNDCLASS wc;  //      

	wc.style         = CS_HREDRAW | CS_VREDRAW; // 
	wc.lpfnWndProc   = WndProc;                 //  
	wc.cbClsExtra    = 0;
	wc.cbWndExtra    = 0;
	wc.hInstance     = hInstance;               // 
	wc.hIcon         = LoadIcon(NULL, IDI_APPLICATION);
	wc.hCursor		  = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = GetStockBrush(LTGRAY_BRUSH);
	wc.lpszMenuName  = "GLMENU";         //   
	wc.lpszClassName = szClassName;      //    

	return RegisterClass(&wc);
}



//   
HWND CreateApp(HINSTANCE hInstance, int nCmdShow)
{
	HWND hwnd;   //   

	hwnd = CreateWindow(szClassName,		//  
						szWindowTitle,		// 
						WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN
						| WS_CLIPSIBLINGS,	// 
						CW_USEDEFAULT,		//   
						CW_USEDEFAULT,
						CW_USEDEFAULT,
						CW_USEDEFAULT,
						NULL,
						NULL,
						hInstance,
						NULL);

	if(hwnd == NULL)
		return hwnd;

// 
	ShowWindow(hwnd,nCmdShow);
// 
	UpdateWindow(hwnd);

	return hwnd;
}

// ,  
LRESULT CALLBACK  WndProc(HWND hwnd, UINT msg,
							WPARAM wParam, LPARAM lParam)
{

	switch(msg)
	{
		case WM_COMMAND:  //    
			switch(wParam)
			{
				case 200:
					InvalidateRect(hwnd,NULL,TRUE); // 
					break;
				case 202:
					DrawOpenGL(hwnd);   //  
					break;
				case 201:
					DestroyWindow(hwnd); //  
					break;
			}
			break;  

		case WM_DESTROY:
			DestroyOpenGL();
			break;
		
		default:
			return DefWindowProc(hwnd,msg,wParam,lParam);
	}

	return 0L;
}

//  
void DestroyOpenGL()
{
	PostQuitMessage(0);   //  
}


//  
void DrawOpenGL(HWND hwnd)
{
	HDC hdc = GetDC(hwnd); //  

	//   GDI
	Rectangle(hdc,100,200,500,250);

	
	//           OpenGL
	
	//   
	PIXELFORMATDESCRIPTOR pfd;

	//    pfd
	memset(&pfd, 0, sizeof(PIXELFORMATDESCRIPTOR));

	//   
	pfd.nSize      = sizeof(PIXELFORMATDESCRIPTOR);
	pfd.nVersion   = 1;
	pfd.dwFlags    = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL;
	pfd.iPixelType = PFD_TYPE_RGBA;
	pfd.cColorBits = 24;

	//    
	int iPixelFormat = ChoosePixelFormat(hdc, &pfd);

	//      
	SetPixelFormat(hdc, iPixelFormat, &pfd);

	//   OpenGL
	HGLRC hglrc = wglCreateContext(hdc);
	//    OpenGL
	wglMakeCurrent(hdc, hglrc);


	//    
	gluOrtho2D(-400, 400, -300, 300);

	//  
	glColor3f(0.0, 1.0, 0.0);

	// 
	glBegin(GL_TRIANGLES);
	glVertex2f(-100, -100);
	glVertex2f(0, 100);
	glVertex2f(100, -100);
	glEnd();

	//  
	glColor3f(1.0, 0.0, 0.0);
	//  - 6 
	glLineWidth(6.0);

	//   
	glBegin(GL_LINE_STRIP);
	glVertex2f(0, 150);
	glVertex2f(150, -130);
	glVertex2f(-150, -130);
	glVertex2f(0, 150);
	glEnd();

	//     OpenGL
	glFinish();
	
	//     
	wglMakeCurrent(NULL, NULL);
	//   OpenGL
	wglDeleteContext(hglrc);

	//OpenGL            

	//   GDI
	Rectangle(hdc,50,220,450,270);

	//  
	ReleaseDC(hwnd,hdc);
}





